@@ -1,5 +1,5 @@
 package Curio;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Curio::Declare qw();
 use Curio::Role qw();
@@ -84,8 +84,6 @@ Then use your new Curio class elsewhere:
     
     my $chi = myapp_cache('geo_ip')->chi();
 
-
-
 =head1 DESCRIPTION
 
 Curio is a toolbox for building a class which holds a resource (or
@@ -101,8 +99,8 @@ library you've got two jobs.
 First, you create classes in your application which use Curio.  You'll
 have one class for each type of resource you want available to your
 application as a whole.  So, for example, you'd have a Curio class for
-your database connections, another for your graphite client, and perhaps
-a third for your CRM client.
+your database connections, another for your graphite client, and
+perhaps a third for your CRM client.
 
 Your second job is to then modify your application to use your Curio
 classes.  If your application uses an existing framework, such as
@@ -125,9 +123,9 @@ The main drive behind creating Curio is threefold.
 To avoid the extra complexity of passing around references of shared
 resources, such as connections to services.  Often times you'll see
 code which passes a connection to a function, which then passes that
-on to another function, which then creates an object with the connection
-passed as an argument, etc.  This is what is being avoided; it's a messy
-way to write code and prone to error.
+on to another function, which then creates an object with the
+connection passed as an argument, etc.  This is what is being avoided;
+it's a messy way to write code and prone to error.
 
 =item 2.
 
@@ -183,15 +181,12 @@ Which is exactly the same as:
     with 'Curio::Role';
     __PACKAGE__->initialize();
 
-If you're not into the declarative interface, or have some
-other reason to switch around this boilerplate, you may copy the
-above and modify to fit your needs rather than using this module
-directly.
-
-Read more about L<Moo> and L<namespace::clean> if you are not
-familiar with them.
-
+If you're not into the declarative interface, or have some other
+reason to switch around this boilerplate, you may copy the above and
+modify to fit your needs rather than using this module directly.
 
+Read more about L<Moo> and L<namespace::clean> if you are not familiar
+with them.
 
 =head1 TOPICS
 
@@ -204,8 +199,8 @@ To get at a Curio object's resource takes a lot of typing by default.
 Creating an export function that wraps this all up is a great way to
 simplify things.  In your Curio class you can set
 L<Curio::Factory/export_function_name> which will create a function,
-create the C<@EXPORT_OK> package variable, and add the new function
-to it.
+create the C<@EXPORT_OK> package variable, and add the new function to
+it.
 
     # In your Curio class.
     export_function_name 'myapp_cache';
@@ -246,6 +241,9 @@ The generated function can be overriden with your own custom function.
         return __PACKAGE__->factory->fetch_curio( @_ )->chi();
     }
 
+Want to do something more advanced?  Don't use any of the methods
+suggested here and just use the exporter module of your choice.
+
 =head2 Caching
 
 Caching is enabled with L<Curio::Factory/does_caching>.
@@ -257,8 +255,8 @@ for a curio object will return the same one as before.  This option
 should almost always be set as it usually provides a huge performance
 increase.
 
-L<Curio::Factory/cache_per_process> extends the
-caching to handle process/thread changes gracefully.
+L<Curio::Factory/cache_per_process> extends the caching to handle
+process/thread changes gracefully.
 
     cache_per_process;
 
@@ -266,9 +264,9 @@ caching to handle process/thread changes gracefully.
 
 Curio supports fetching curio objects by key.  This is an optional
 feature and by default is turned off.  To turn it on you set
-L<Curio::Factory/does_keys> or just start adding keys
-with L<Curio::Factory/add_key> which will automatically turn
-on C<does_keys>.
+L<Curio::Factory/does_keys> or just start adding keys with
+L<Curio::Factory/add_key> which will automatically turn on
+C<does_keys>.
 
 When keys are enabled a curio class is able to produce different
 objects based on the key.  For example, lets say you have two
@@ -294,24 +292,24 @@ You can also set L<Curio::Factory/default_key>.
 
     default_key 'db1';
 
-Curio objects, by default, have no way of knowing what key was used
-to make them.  If you need to know what key was used to fetch a curio
+Curio objects, by default, have no way of knowing what key was used to
+make them.  If you need to know what key was used to fetch a curio
 object you can set L<Curio::Factory/key_argument>.
 
     key_argument 'key';
     has key => ( is=>'ro' );
 
-The L<Curio::Factory/default_arguments> option can
-be useful when you are not using Moo attributes but still need to set
-defaults for arguments.
+The L<Curio::Factory/default_arguments> option can be useful when you
+are not using Moo attributes but still need to set defaults for
+arguments.
 
     default_arguments ( username => 'dbuser' );
 
 =head2 The Registry
 
 The registry is a lookup table holding memory addresses of resource
-objects pointing at references to curio objects.  What this means
-is, if L<Curio::Factory/does_registry> is set, you can use
+objects pointing at references to curio objects.  What this means is,
+if L<Curio::Factory/does_registry> is set, you can use
 L<Curio::Role/find_curio> to retrieve the curio object for a given
 resource object.
 
@@ -358,8 +356,8 @@ instead use L<Curio::Role/inject_with_guard>.
         'geo_ip', $mock,
     );
 
-When the guard object goes out of scope C<uninject> will be
-called automatically.
+When the guard object goes out of scope C<uninject> will be called
+automatically.
 
 =head2 Singletons
 
@@ -367,19 +365,137 @@ Creating a singleton class is super simple.
 
     package MyApp::Context;
     
+    use Types::Common::String qw( SimpleStr );
+    
     use Curio;
+    use strictures 2;
     
-    use Exporter qw( import );
-    our @EXPORT = qw( myapp_cache );
+    export_function_name 'myapp_context';
+    always_export;
     
-    sub myapp_context {
-        return __PACKAGE__->fetch( @_ );
+    has install_dir => (
+        is  => 'lazy',
+        isa => SimpleStr,
+    );
+    
+    sub _build_install_dir {
+        my $dir = $ENV{MYAPP_INSTALL_DIR};
+        return $dir if $dir;
+        
+        return '/myapp';
     }
     
-    has user_id => ( is=>'rw' );
+    has config_file => (
+        is  => 'lazy',
+        isa => SimpleStr,
+    );
     
-    # Elsewhere:
-    my $current_user_id = myapp_context()->user_id();
+    sub _build_config_file {
+        my ($self) = @_;
+        
+        my $file = $ENV{MYAPP_CONFIG_FILE};
+        return $file if $file;
+        
+        return $self->install_dir() . '/config.ini';
+    }
+
+See L</Configuration> and L</Secrets> for more singleton examples.
+
+=head2 Configuration
+
+Application configuration is one of those systems which can benefit a
+lot from being wrapped up in a curio class.
+
+    package MyApp::Config;
+    
+    use Config::INI::Reader;
+    use MyApp::Context;
+    use Types::Common::String qw( SimpleStr );
+    use Types::Standard qw( HashRef );
+    
+    use Curio;
+    use strictures 2;
+    
+    export_function_name 'myapp_config';
+    always_export;
+    export_resource;
+    resource_method_name 'config';
+    does_caching;
+    
+    has file => (
+        is  => 'lazy',
+        isa => SimpleStr,
+    );
+    
+    sub _build_file {
+        return myapp_context()->config_file();
+    }
+    
+    has config => (
+        is  => 'lazy',
+        isa => HashRef,
+    );
+    
+    sub _build_config {
+        my ($self) = @_;
+        
+        return Config::INI::Reader->read_file(
+            $self->file(),
+        );
+    }
+
+This curio-based configuration class could then be used from anywhere
+to access your application's configuration.
+
+    use MyApp::Config;
+    
+    print myapp_config()->{login_rate_limit};
+
+See L</Singletons> for a working example of C<MyApp::Context>.
+
+=head2 Secrets
+
+Handling secrets requires careful planning and prior experience to get
+right.  It is very easy to leak secrets into logs, onto HTTP error
+pages, emails, and other locations.  Start off right and you can avoid
+these issues in a lot of common cases by adhering to one important
+rule: never store your secrets in objects as plain string values.
+
+Whenever you need a secret, request it from your secret storage.  Your
+secret storage may just be a configuration file initialy, and thats ok
+when you are just starting, but once your project starts being used by
+more than other developers and testers you'll need to consider a more
+robust solution, such as L<Vault|https://www.vaultproject.io/> or
+L<Kubernetes secret objects|https://kubernetes.io/docs/concepts/configuration/secret/>.
+
+The important thing is that your API for accessing secrets remain
+consistant even as you transition from one secret storage to another.
+Wrapping your secret storage in a curio class can make these sorts of
+transitions run smoothly as only the code in your curio class needs to
+be changed.
+
+    package MyApp::Secrets;
+    
+    use MyApp::Service::Vault;
+    
+    use Curio;
+    use strictures 2;
+    
+    export_function_name 'myapp_secret';
+    always_export;
+    does_caching;
+    
+    sub myapp_secret {
+        my $key = pop;
+        return myapp_vault()->secret( path=>$key )->data->{password};
+    }
+
+This generic interface provided by your curio class would not need
+changing if you completely moved your secret backend to a different
+system.
+
+    use MyApp::Secrets;
+    my $db_password = myapp_secret('db-main-www');
 
 =head2 Handling Arguments
 
@@ -389,25 +505,18 @@ Creating a singleton class is super simple.
 
 =head2 Custom Curio Roles
 
-=head2 Secrets
-
-=head2 Configuration
-
-
-
 =head1 IMPORTANT PRACTICES
 
 =head2 Avoid Holding onto Curio Objects and Resources
 
-Curio is designed to make it cheap to retrieve Curio objects
-and the underlying resources.  Take advantage of this.  Don't
-pass around your resource objects or put them in attributes.
-Instead, when you need them, get the from your Curio classes.
+Curio is designed to make it cheap to retrieve Curio objects and the
+underlying resources.  Take advantage of this.  Don't pass around your
+resource objects or put them in attributes.  Instead, when you need
+them, get the from your Curio classes.
 
-If your Curio class supports keys, then passing around the
-key that you want particular code to be using, rather than the
-Curio object or the resource, is a much better way of handling
-things.
+If your Curio class supports keys, then passing around the key that
+you want particular code to be using, rather than the Curio object or
+the resource, is a much better way of handling things.
 
 Read more of the reasoning for this in L<Curio/MOTIVATION>.
 
@@ -415,10 +524,11 @@ Read more of the reasoning for this in L<Curio/MOTIVATION>.
 
 It is tempting to use the L<Curio/INTEGRATIONS> such as
 L<Catalyst::Model::Curio>, and sometimes it is necessary to do so.
-Most of the time there is no need to add that extra layer of complexity.
+Most of the time there is no need to add that extra layer of
+complexity.
 
-Using Catalyst as an example, there are few reasons you can't
-just use your Curio classes directly from your Catalyst controllers.
+Using Catalyst as an example, there are few reasons you can't just use
+your Curio classes directly from your Catalyst controllers.
 
 At ZipRecruiter, where we have some massive Catalyst applications, we
 only use Catalyst models in the few cases where other parts of
@@ -428,17 +538,14 @@ easier to deal with.
 
 =head2 Appropriate Uses of Key Aliases
 
-Key aliases are meant as a tool for migrating and merging keys.
-They are meant to be something you temporarily setup as you change
-your code to use the new keys, and then once done you remove the
-aliases.
-
-It can be tempting to use key aliases to provide simpler or alternative
-names for existing keys.  The problem with doing this is now you've
-introduced multiple keys for the same Curio class which in practice
-causes unnecessary confusion.
-
+Key aliases are meant as a tool for migrating and merging keys.  They
+are meant to be something you temporarily setup as you change your
+code to use the new keys, and then once done you remove the aliases.
 
+It can be tempting to use key aliases to provide simpler or
+alternative names for existing keys.  The problem with doing this is
+now you've introduced multiple keys for the same Curio class which in
+practice causes unnecessary confusion.
 
 =head1 ROLES
 
@@ -467,8 +574,8 @@ L<Curio::Role::GitLab::API::v4>
 
 =head1 INTEGRATIONS
 
-The CPAN modules listed here integrate Curio with other things
-such as web frameworks.
+The CPAN modules listed here integrate Curio with other things such as
+web frameworks.
 
 =over
 
@@ -480,8 +587,6 @@ L<Catalyst::Model::Curio>
 
 On a related note, take a look at L<Curio/Use Curio Directly>.
 
-
-
 =head1 SEE ALSO
 
 It is hard to find anything out there on CPAN which is similar to
@@ -495,14 +600,12 @@ are baked into the framework.  The idea is similar though.
 
 Someone started something that looks vaguely similar called L<Trinket>
 (this was one of the names I was considering and found it by accident)
-but it never got any love since initial release in 2012 and is incomplete.
+but it never got any love since initial release in 2012 and is
+incomplete.
 
 Since Curio can do singletons, you may want to check out
 L<MooX::Singleton> and L<MooseX::Singleton>.
 
-
-
-
 =head1 SUPPORT
 
 Please submit bugs and feature requests to the
@@ -512,10 +615,10 @@ L<https://github.com/bluefeet/Curio/issues>
 
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to L<ZipRecruiter|https://www.ziprecruiter.com/>
-for encouraging their employees to contribute back to the open
-source ecosystem.  Without their dedication to quality software
-development this distribution would not exist.
+Thanks to L<ZipRecruiter|https://www.ziprecruiter.com/> for
+encouraging their employees to contribute back to the open source
+ecosystem.  Without their dedication to quality software development
+this distribution would not exist.
 
 =head1 AUTHORS
 
